#include "../../include/package.h"
#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <stdlib.h>
#include <string.h>

struct __lm_ctx_update_cb_data {
  lm_ctx_update_callback_t callback;
  lm_ctx_update_state_t state;
  void *data;
};

bool __lm_ctx_update_callback(lm_ctx_t *ctx, lm_pkg_t *pkg, char *file, size_t current, size_t total, void *data){
  struct __lm_ctx_update_cb_data *cbdata = data;

  if(NULL == cbdata->callback)
    return true;

  return cbdata->callback(ctx, pkg, cbdata->state, file, current, total, cbdata->data);
}

lm_ctx_update_list_t *lm_ctx_update_list(lm_ctx_t *ctx){
  if(NULL == ctx){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_ctx_update_list_t *list = malloc(sizeof(lm_ctx_update_list_t));
  lm_pkg_t cur, *new = NULL;
  
  bzero(list, sizeof(lm_ctx_update_list_t));
  lm_package_init(&cur);

  while(lm_ctx_database_next(ctx, &cur)){
    if((new = lm_ctx_pool_find(ctx, cur.name, NULL)) == NULL)
      continue;

    if(eq(new->version, cur.version))
      continue;

    if(NULL == list->packages)
      list->packages = malloc(sizeof(lm_pkg_t));
    else
      list->packages = realloc(list->packages, (list->count+1)*sizeof(lm_pkg_t));
    lm_package_copy(&list->packages[list->count++], &cur);
  }

  lm_ctx_database_next_free(ctx, &cur);
  return list;
}

lm_pkg_t *lm_ctx_update_list_next(lm_ctx_update_list_t *list){
  if(NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  if(NULL == list->packages)
    return NULL;

  if(NULL == list->cur)
    list->cur = &list->packages[0];

  for(int i = 0; i < list->count-1; i++){
    if(list->cur != &list->packages[i])
      continue;
    list->cur = &list->packages[i+1];
    return list->cur;
  }

  list->cur = NULL;
  return list->cur;
}

void lm_ctx_update_list_free(lm_ctx_update_list_t *list){
  if(NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return;
  }

  if(NULL != list->packages){
    for(int i = 0; i < list->count; i++)
      lm_package_free(&list->packages[i]);
    free(list->packages);
  }
 
  free(list);
}

bool lm_ctx_update(lm_ctx_t *ctx, lm_pkg_t *pkg, lm_ctx_update_callback_t callback, void *data){
  if(NULL == ctx || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  struct __lm_ctx_update_cb_data cbdata = {
    .state = UPDATE_REMOVE,
    .callback = callback,
    .data = data,
  };
  lm_pkg_t *new = NULL;

  if((new = lm_ctx_pool_find(ctx, pkg->name, NULL)) == NULL){
    lm_error_set(LM_ERR_PkgNotFound);
    return false;
  }

  if(eq(new->version, pkg->version)){
    lm_error_set(LM_ERR_PkgUpToDate);
    return false;
  }

  if(!lm_ctx_remove(ctx, pkg, __lm_ctx_update_callback, &cbdata))
    return false; // error set by function
  
  if(!lm_ctx_install(ctx, new, __lm_ctx_update_callback, &cbdata))
    return false; // error set by function

  return true;
}
